package cn.fastfun.controller.api;

import cn.fastfun.controller.param.LoginParam;
import cn.fastfun.service.AlarmService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.AlarmCode;
import cn.fastfun.service.entity.SysUser;
import cn.fastfun.util.FileUtils;
import com.alibaba.excel.EasyExcel;
import com.bridge.dto.*;
import com.bridge.service.JpaService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.util.DigestUtils;
import org.springframework.util.ObjectUtils;
import org.springframework.util.StringUtils;
import org.springframework.validation.BindingResult;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@Api(tags = {"4.0 故障字典"})
@RestController
@RequestMapping("/api/v1/alarm/code")
public class AlarmCodeController {

    @Autowired
    UtilService utilService;

    @Resource(name = "alarmCodeService")
    JpaService<AlarmCode, String> alarmCodeService;

    @Resource
    AlarmService alarmService;

    @ApiOperation(value = "导入Excel")
    @PostMapping("/excel/upload")
    public ApiDTO upload(MultipartFile file, String protocolCode) {
        if (ObjectUtils.isEmpty(protocolCode) || "undefined".equals(protocolCode)) {
            return ApiDTO.error("协议不能为空");
        }
        String filePath = alarmService.uploadFile(file);
        List<String> ids = alarmCodeService.findAll(QueryParamExp.eq("protocolCode", protocolCode)).stream().map(p -> p.getId()).collect(Collectors.toList());
        if (!CollectionUtils.isEmpty(ids)) {
            alarmCodeService.delete(ids);
        }
        List<Map<Integer, String>> listMap = EasyExcel.read(alarmService.getPath().concat(filePath)).sheet().headRowNumber(1).doReadSync();
        String title = "";
        List<AlarmCode> list = new ArrayList<>();
        for (Map<Integer, String> m : listMap) {
            if (ObjectUtils.isEmpty(m.get(0))) {
                m.put(0, title);
            } else {
                title = m.get(0);
            }
            AlarmCode p = new AlarmCode(m);
            p.setProtocolCode(protocolCode);
            p.setFileUrl(filePath);
            list.add(p);
        }
        if (!CollectionUtils.isEmpty(list)) {
            alarmCodeService.save(list);
        }
        return ApiDTO.ok();
    }

    /**
     * 数据保存
     */
    @ApiOperation(value = "文件流")
    @GetMapping(value = "excel/{protocol}")
    public ApiDTO save(@PathVariable String protocol, HttpServletResponse response) {
        List<AlarmCode> list = alarmCodeService.findAll(QueryParamExp.eq("protocolCode", protocol));
        File file = new File(alarmService.getPath().concat(list.get(0).getFileUrl()));
        FileUtils.outFile(response, list.get(0).getFileUrl(), file);
        return ApiDTO.ok("保存成功");
    }

    @ApiOperation(value = "分页搜索")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO pageQuery(@RequestBody QueryParam param) {
        param.addGroup("protocolCode");
        return new ApiPageDTO(null, alarmCodeService.findByParam(param));
    }
}
