package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.DevicePageStatDTO;
import cn.fastfun.controller.param.*;
import cn.fastfun.service.*;
import cn.fastfun.service.entity.AppDevice;
import cn.fastfun.service.entity.AppDeviceLog;
import cn.fastfun.service.entity.PackModel;
import cn.fastfun.util.DateUtils;
import cn.fastfun.util.ObjectUtil;
import cn.fastfun.util.VerifyUtil;
import com.bridge.dto.*;
import com.bridge.exception.ApiRuntimeException;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.BooleanUtils;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.springframework.dao.EmptyResultDataAccessException;
import org.springframework.data.domain.Page;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.util.StringUtils;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.persistence.criteria.CriteriaBuilder;
import javax.servlet.http.HttpServletResponse;
import java.math.BigInteger;
import java.text.SimpleDateFormat;
import java.util.Arrays;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @author Bridge AutoGen
 */
@Slf4j
@Api(tags = {"2.0 设备信息"})
@RestController
@RequestMapping("/api/v1/appdevice")
public class ApiAppDeviceController {
    //业务类
    @Resource(name = "appDeviceService")
    AppDeviceService appDeviceService;

    @Resource
    UtilService utilService;

    @Resource
    AppDeviceLogService appDeviceLogService;

    @Resource
    SysExcelFieldService sysExcelFieldService;

    @Resource
    JdbcTemplate jdbcTemplate;

    @Resource
    PackModelService packModelService;

    /**
     * 数据保存
     */

    @ApiOperation(value = "新增电池")
    @PostMapping("save")
    public ApiDTO save(@RequestBody @ApiParam(name = "设备信息对象", value = "传入json格式", required = true) DeviceFormParam param) {
        // 此处定义的sn实际上是指的表中的imei，也就是实体类的属性imei
        String checkSnResult = VerifyUtil.checkImei(param.getImei());

        List<AppDevice> checkDuplicatedResultOnImei = appDeviceService.findAll(
                QueryParamExp.eq("imei", param.getImei()));
        if (!checkDuplicatedResultOnImei.isEmpty()) {
            return ApiDTO.error(500, "该imei已经被录入", checkDuplicatedResultOnImei);
        }
        List<AppDevice> checkDuplicatedResultOnSn = appDeviceService.findAll(
                QueryParamExp.eq("sn", param.getSn()));
        if (!checkDuplicatedResultOnSn.isEmpty()) {
            return ApiDTO.error(500, "该sn已经被录入", checkDuplicatedResultOnSn);
        }
        // 检查该sn号是否产生过数据
        Map<String, Object> bmsData = new HashMap<>();
        Map<String, Object> gpsData = new HashMap<>();
        try {
            bmsData = jdbcTemplate.queryForMap("select devcode from ff_battery_status where devcode='" + param.getSn() + "'");
        } catch (EmptyResultDataAccessException e) {
            bmsData.put("devcode", null);
        }
        try {
            gpsData = jdbcTemplate.queryForMap("select devcode from ff_location where devcode='" + param.getSn() + "'");
        } catch (EmptyResultDataAccessException e) {
            gpsData.put("devcode", null);
        }
        //pack和cell 校验
        if (!org.apache.commons.lang3.StringUtils.isNotBlank(param.getPackModel())) {
            throw new ApiRuntimeException("包型号缺失");
        }
        if (!org.apache.commons.lang3.StringUtils.isNotBlank(param.getCellModel())) {
            throw new ApiRuntimeException("单体型号缺失");
        }

        if (!param.getImei().substring(1, 3).equals(param.getPackModel().substring(0, 2))) {
            throw new ApiRuntimeException("包型号与imei对应关系错误");
        }
        if (!param.getImei().substring(3, 6).equals(param.getCellModel().substring(0, 3))) {
            throw new ApiRuntimeException("单体型号与imei对应关系错误");
        }
        PackModel packModel = packModelService.getOne(Arrays.asList(QueryParamExp.eq("cellModel", param.getCellModel()),
                QueryParamExp.eq("packModel", param.getPackModel())));
        if (packModel == null) {
            throw new ApiRuntimeException("包型号和单体型号配置错误");
        }

//        if (StringUtils.isEmpty(bmsData.get("devcode")) && StringUtils.isEmpty(gpsData.get("devcode"))) {
//            return ApiDTO.error(500, "该电池未产生过数据，禁止录入");
//        }
        if (checkSnResult.equals("passed")) {
            // 查询当前操作id的最大值
            StringBuffer sql = new StringBuffer();
            sql.append("SELECT COALESCE(MAX(operate_id),0) as max_opid from app_device where operate_id is not null");
            log.info("SQL: {}", sql.toString());
            ApiPageDTO page = appDeviceLogService.getListBySQL(sql.toString(), new HashMap<>(), new QueryParam());
            List<Map<String, Object>> data = (List<Map<String, Object>>) page.getData();
            BigInteger operateID = (BigInteger) data.get(0).get("max_opid");
            operateID = operateID.add(BigInteger.valueOf(1));
            // 实体
            AppDevice entity = new AppDevice().fromFormParam(param);
            entity.setOperateID(operateID.intValue());
            entity.setOperator(utilService.getUserName());
            entity.setPackModel(param.getPackModel());
            entity.setCellModel(param.getCellModel());
            appDeviceService.save(entity);
            return ApiDTO.ok("保存成功");
        } else {
            return ApiDTO.error(500, checkSnResult);
        }
    }

    @ApiOperation(value = "电池自检")
    @RequestMapping(value = "selfCheck", method = RequestMethod.POST)
    public ApiDTO selfCheck(@RequestBody BenchQueryDetailParam param) {
        AppDevice appDevice = appDeviceService.getOne(QueryParamExp.eq("sn", param.getSn()));
        if (!(appDevice.getCheckStatus().equals(1))) {
            Map<String, Boolean> checkRes = appDeviceLogService.selfCheck(param.getSn());
            appDevice.setCheckStatus(BooleanUtils.toInteger(checkRes.get("checkResult")));
            appDevice.setCheckStatusDataConnect(BooleanUtils.toInteger(checkRes.get("checkDataConnectResult")));
            appDevice.setCheckStatusLocation(BooleanUtils.toInteger(checkRes.get("checkLocationResult")));
            appDevice.setCheckStatusLock(BooleanUtils.toInteger(checkRes.get("checkLockResult")));
            appDevice.setCheckStatusFault(BooleanUtils.toInteger(checkRes.get("checkFaultResult")));
            appDevice.setCheckStatusVoltage(BooleanUtils.toInteger(checkRes.get("checkVoltageResult")));
            appDeviceService.save(appDevice);
        }
        return ApiDTO.ok("200", appDevice.getCheckStatus());
    }

    @ApiOperation(value = "导入模板下载")
    @GetMapping("excelTemplate")
    public void excelTemplate(HttpServletResponse response, @RequestParam String name) {
        HSSFWorkbook workbook = sysExcelFieldService.getTemplate("invoice");
        sysExcelFieldService.outExcelFile(workbook, response, name);
    }

    @ApiOperation(value = "批量新增电池")
    @RequestMapping(value = "excelImport", method = RequestMethod.POST)
    public ApiDTO excelImport(MultipartFile file) {
        appDeviceService.importDevice(sysExcelFieldService.importExcel("invoice", file));
        return ApiDTO.ok();
    }

    @ApiOperation(value = "设备统计")
    @RequestMapping(value = "count", method = RequestMethod.POST)
    public ApiDTO deviceCountStat() {

        DevicePageStatDTO dto = new DevicePageStatDTO();
        Long deviceCount = appDeviceService.getCount();
        Long notInStorage = appDeviceService.getCount(QueryParamExp.eq("status", 0));
        Long inStorage = deviceCount - notInStorage;
        Long transfer = appDeviceService.getCount(QueryParamExp.eq("status", 2));
        Long transferAble = appDeviceService.getCount(QueryParamExp.eq("status", 1));
        Long outStorage = appDeviceService.getCount(QueryParamExp.eq("status", 3));
        Long handle = appDeviceService.getCount(QueryParamExp.eq("status", 4));
        dto.setTotal(deviceCount);
        dto.setNotInStorage(notInStorage);
        dto.setInStorage(inStorage);
        dto.setTransfer(transfer);
        dto.setTransferAble(transferAble);
        dto.setOutStorage(outStorage);
        dto.setOutStorageAble(transfer);
        dto.setHandle(handle);
        return ApiDTO.ok(dto);
    }

    @ApiOperation(value = "设备录入 搜索 & 重置")
    @RequestMapping(value = "pageQuery", method = RequestMethod.POST)
    public ApiPageDTO batchNumPageQuery(@RequestBody DeviceBatchQueryParam param) {

        StringBuffer sql = new StringBuffer();
        sql.append("select t1.batch_num,t1.sn,t1.imei,t1.operate_id, t1.operator, t1.deliver_time,count(0) as total,count(if(status = 1,true,null)) as in_storage,count(if(status = 2,true,null)) as transfer,");
        sql.append("count(if(status = 3,true,null)) as out_storage,count(if(status = 4,true,null)) as handle,");
        sql.append("t1.add_time,t1.received_place from app_device t1");

        Integer conditionCount = new Integer(0);
        if (!StringUtils.isEmpty(param.getBatchNum())) {
            sql.append(conditionCount == 0 ? " where" : " and").append(" t1.batch_num like '%").append(param.getBatchNum()).append("%'");
            conditionCount++;
        }

        if (!StringUtils.isEmpty(param.getRecordTimeStart())) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            sql.append(conditionCount == 0 ? " where" : " and").append(" t1.add_time >= '").append(sdf.format(param.getRecordTimeStart())).append("'");
            conditionCount++;
        }

        if (!StringUtils.isEmpty(param.getRecordTimeEnd())) {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
            sql.append(conditionCount == 0 ? " where" : " and").append(" t1.add_time <= '").append(sdf.format(param.getRecordTimeEnd())).append("'");
            conditionCount++;
        }

        sql.append(" group by t1.operate_id");
        if (!StringUtils.isEmpty(param.getOrderBy())) {
            sql.append(param.getOrderBy());
        }

        log.info("SQL: {}", sql.toString());


        ApiPageDTO page = appDeviceService.getListBySQL(sql.toString(), new HashMap<>(), param);
        if (null != page.getData()) {
            // 组装数据
            List<Map<String, Object>> list = (List<Map<String, Object>>) page.getData();
            for (Map<String, Object> p : list) {
                AppDevice device = new AppDevice();
                device.setImei(ObjectUtil.obj2String(p.get("imei")));
                appDeviceService.addImeiTitle(device);

                p.put("type_title", device.getTypeTitle());
                p.put("pack_title", device.getPackTitle());
                p.put("deliver_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("deliver_time")), DateUtils.YMD));
                p.put("add_time", DateUtils.toString(ObjectUtil.obj2Date(p.get("add_time")), DateUtils.YMDHMS));
            }
            page.setData(list);
        }
        return page;
    }

    @ApiOperation(value = "查看详情 --> 搜索 & 重置")
    @RequestMapping(value = "pageQuery/detail", method = RequestMethod.POST)
    public ApiPageDTO devicePageQueryDetial(@RequestBody DeviceOperateQueryParam param) {

        Page<AppDevice> devicePage = appDeviceService.findByParam(param);
        devicePage.getContent().forEach(p ->
                appDeviceService.addImeiTitle(p)
        );
        return new ApiPageDTO(null, devicePage);
    }

    @ApiOperation(value = "根据sn 查询相关信息")
    @RequestMapping(value = "pageQuery/bySN", method = RequestMethod.POST)
    public Map deviceSnPageQuery(@RequestBody SnQueryParam param) {
        QueryParam query = new QueryParam();
        query.addParam(QueryParamExp.desc("addTime"));
        query.addParam(QueryParamExp.eq("type", 2));
        query.addParam(QueryParamExp.eq("sn", param.getSn()));
        Page<AppDeviceLog> deviceLogPage = appDeviceLogService.findByParam(query);
        AppDeviceLog lastDeviceLogPage = new AppDeviceLog();
        if (!deviceLogPage.isEmpty()) {
            lastDeviceLogPage = deviceLogPage.getContent().get(0);
        }


        AppDevice device = appDeviceService.getOne(new QueryParamExp("sn", param.getSn()));

        appDeviceService.addImeiTitle(device);
        lastDeviceLogPage.appendFormDevice(device);
        Map<String, Object> res = new HashMap<>();
        res.put("sn", param.getSn());
        res.put("pack_title", lastDeviceLogPage.getPackTitle());
        res.put("chem_title", device.getSpecTitle());
        res.put("type_title", device.getTypeTitle());
        res.put("device_describe", lastDeviceLogPage.getTfDescribe());
        res.put("device_use", lastDeviceLogPage.getTfUsed());
        res.put("device_type", lastDeviceLogPage.getTfDeviceType());
        res.put("device_owner", device.getOwnerId());
        res.put("transfer_time", device.getTransferTime());
        res.put("outstorage_time", device.getOutstorageTime());
        return res;
    }


}