package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.CellDto;
import cn.fastfun.controller.param.QueryParamForCellModel;
import cn.fastfun.service.CellModelService;
import cn.fastfun.service.vo.CellModelVo;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;

/**
 * @author zyg
 * @className CellModelController
 * @date 2023年02月16日 11:41
 */
@Api(tags = "单体型号")
@RestController
@RequestMapping("/api/v1/cellModel")
public class CellModelController {

    private static final String OK = "OK";

    @Resource
    CellModelService cellModelService;

    @ApiOperation(value = "分页搜索单体型号")
    @PostMapping("/queryCell")
    public ApiDTO getCellModelPageInfo(@RequestBody QueryParamForCellModel param) {
        ApiPageDTO result = cellModelService.getCellModelPageInfo(param);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "单体型号详情")
    @GetMapping("/cellDetail/{id}")
    public ApiDTO getCellDetail(@PathVariable("id") String id) {
        CellModelVo result = cellModelService.getCellDetail(id);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "删除单体型号")
    @DeleteMapping("/delCell/{id}")
    public ApiDTO delCell(@PathVariable("id") String id) {
        String result = cellModelService.delCell(id);
        if (result.equals(OK)) {
            return ApiDTO.ok("删除成功！", result);
        }else{
            return ApiDTO.error(500, result);
        }
    }

    @ApiOperation(value = "新增单体型号")
    @PostMapping("/addCell")
    public ApiDTO addCell(@RequestBody CellDto dto) {
        String result = cellModelService.addCell(dto);
        if (result.equals(OK)) {
            return ApiDTO.ok("新增成功！", result);
        }else{
            return ApiDTO.error(500, result);
        }
    }

    @ApiOperation(value = "修改单体型号")
    @PostMapping("/modifyCell")
    public ApiDTO modifyCell(@RequestBody CellDto dto) {
        String result = cellModelService.modifyCell(dto);
        if (result.equals(OK)) {
            return ApiDTO.ok("修改成功！", result);
        }else{
            return ApiDTO.error(500, result);
        }
    }
}
