package cn.fastfun.controller.api;

import cn.fastfun.controller.dto.PackDto;
import cn.fastfun.controller.param.QueryParamForPackModel;
import cn.fastfun.service.PackModelService;
import cn.fastfun.service.vo.PackModelReadyListVo;
import cn.fastfun.service.vo.PackModelVo;
import com.bridge.dto.ApiDTO;
import com.bridge.dto.ApiPageDTO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;

/**
 * @author zyg
 * @className PackModelController
 * @date 2023年02月16日 11:41
 */
@Api(tags = "包型号")
@RestController
@RequestMapping("/api/v1/packModel")
public class PackModelController {

    private static final String OK = "OK";

    @Resource
    PackModelService packModelService;

    @ApiOperation(value = "分页搜索包型号")
    @PostMapping("/queryPack")
    public ApiDTO getPackModelPageInfo(@RequestBody QueryParamForPackModel param) {
        ApiPageDTO result = packModelService.getPackModelPageInfo(param);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "包型号详情")
    @GetMapping("/packDetail/{id}")
    public ApiDTO getPackDetail(@PathVariable("id") String id) {
        PackModelVo result = packModelService.getPackDetail(id);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "删除包型号")
    @DeleteMapping("/delPack/{id}")
    public ApiDTO delPack(@PathVariable("id") String id) {
        String result = packModelService.delPack(id);
        if (result.equals(OK)) {
            return ApiDTO.ok("删除成功！", result);
        } else {
            return ApiDTO.error(500, result);
        }
    }

    @ApiOperation(value = "新增包型号")
    @PostMapping("/addPack")
    public ApiDTO addPack(@RequestBody PackDto dto) {
        String result = packModelService.addPack(dto);
        if (result.equals(OK)) {
            return ApiDTO.ok("新增成功！", result);
        } else {
            return ApiDTO.error(500, result);
        }
    }

    @ApiOperation(value = "修改包型号")
    @PostMapping("/modifyPack")
    public ApiDTO modifyPack(@RequestBody PackDto dto) {
        String result = packModelService.modifyPack(dto);
        if (result.equals(OK)) {
            return ApiDTO.ok("修改成功！", result);
        } else {
            return ApiDTO.error(500, result);
        }
    }

    @ApiOperation(value = "页面加载单体型号列表")
    @GetMapping("/ready")
    public ApiDTO readyForListBox() {
        PackModelReadyListVo result = packModelService.readyForListBox();
        return ApiDTO.ok("修改成功！", result);
    }

    @ApiOperation(value = "通过包型号获取单体型号列表")
    @GetMapping("/cellModelByPack/{packModel}")
    public ApiDTO getCellModelByPack(@PathVariable("packModel") String packModel) {
        PackModelReadyListVo result = packModelService.getCellModelByPack(packModel);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "通过单体型号获取包型号列表")
    @GetMapping("/packModelByCell/{cellModel}")
    public ApiDTO getPackModelByCell(@PathVariable("cellModel") String cellModel) {
        List<PackModelVo> result = packModelService.getPackModelByCell(cellModel);
        return ApiDTO.ok("查询成功！", result);
    }

    @ApiOperation(value = "包型号列表")
    @GetMapping("/packModelList")
    public ApiDTO getPackModelList() {
        List<PackModelVo> result = packModelService.getPackModelList();
        return ApiDTO.ok("查询成功！", result);
    }
}
