package cn.fastfun.controller.api;


import cn.fastfun.service.AppDeviceService;
import cn.fastfun.service.ProductService;
import cn.fastfun.service.entity.IdParamForStatus;
import cn.fastfun.service.entity.Product;
import cn.fastfun.service.entity.QueryParamForProduct;
import cn.fastfun.util.VerifyUtil;
import com.bridge.dto.*;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.*;
import java.util.stream.Collectors;

@Api(tags = "设备类型记录（典表）")
@RestController
@RequestMapping("/api/v1/product")
public class ProductController {

    @Resource
    ProductService productService;
    @Resource
    AppDeviceService appDeviceService;

    @ApiOperation(value = "保存SN规则信息")
    @PostMapping("/save")
    public ApiDTO save(@RequestBody @ApiParam(name = "SN规则对象", value = "传入json格式", required = true) Product product) {
        //已经使用过的imei对应的值不可以更改 M GM CL N 7 50 M 214L028,未被使用过的可以随便改
        String sql = "select id from app_device where imei is not null ";
        String condition = "";
        if (product.getId() != null) {
            Product p = productService.getOne(QueryParamExp.eq("id", product.getId()));
            if (product.getSnTypeEn() != null) {
                switch (product.getSnTypeEn()) {
                    //(1位)=产品类型代码
                    case 1:
                        condition = "and substring(imei,1,1)='" + p.getSnTypeCode() + "'";
                        break;
                    //(2-3位)=PACK制造商简称
                    case 2:
                        condition = "and substring(imei,2,2)='" + p.getSnTypeCode() + "'";
                        break;
                    //(4-5位)=电芯厂家名称
                    case 3:
                        condition = "and substring(imei,4,2)='" + p.getSnTypeCode() + "'";
                        break;
                    //(6位)=产品规格代码 L化学成分
                    case 4:
                        condition = "and substring(imei,6,1)='" + p.getSnTypeCode() + "'";
                        break;
                    //(7位)=电压
                    case 5:
                        condition = "and substring(imei,7,1)='" + p.getSnTypeCode() + "'";
                        break;
                    //(8-9位)=电量
                    case 6:
                        condition = "and substring(imei,8,2)='" + p.getSnTypeCode() + "'";
                        break;
                    //(10位)=产品扩展代码
                    case 7:
                        condition = "and substring(imei,10,1)='" + p.getSnTypeCode() + "'";
                        break;
                    default:
                        condition = "";
                }
                if (StringUtils.isNotBlank(condition)) {
                    sql = sql.concat(condition);
                }

                if (!p.getSnTypeCode().equals(product.getSnTypeCode())) {
                    List<Map<String, Object>> mapList = appDeviceService.getListBySQL(sql, new HashMap<>());
                    if (mapList.size() > 0) {
                        return ApiDTO.error(500, "代码 " + p.getSnTypeCode() + " 已经被使用，不允许修改");
                    }
                }
            }
        }
        // 校验输入的SN类型对应的编号
        String checkSnTypeCodeResult = VerifyUtil.checkSnTypeCode(product);
        List<Product> checkDuplicatedResult = productService.findAll(
                Arrays.asList(QueryParamExp.eq("snTypeCode", product.getSnTypeCode()),
                        QueryParamExp.eq("snType", product.getSnType())));
        if (!checkDuplicatedResult.isEmpty()) {
            if (product.getId() != null) {
                List<Product> collect = checkDuplicatedResult.stream()
                        .filter(o -> !o.getId().equals(product.getId()))
                        .collect(Collectors.toList());
                if (!collect.isEmpty()) {
                    return ApiDTO.error(500, "该规则行已存在", checkDuplicatedResult);
                }
            } else {
                return ApiDTO.error(500, "该规则行已存在", checkDuplicatedResult);
            }
        }
        if (checkSnTypeCodeResult.equals("passed")) {
            Product tmp = productService.save(product);
            return ApiDTO.ok("保存成功", tmp);
        } else {
            return ApiDTO.error(500, checkSnTypeCodeResult);
        }
    }

    @ApiOperation(value = "查询SN规则信息")
    @PostMapping("/get")
    public ApiDTO get(@RequestBody @ApiParam(name = "SN规则行对应的id", required = true) IdParam param) {
        return ApiDTO.ok("查询成功！", productService.get(param.getId()));
    }

    @ApiOperation(value = "删除SN规则信息")
    @PostMapping("/delete")
    public ApiDTO delete(@RequestBody @ApiParam(name = "SN规则行对应的id", required = true) IdParamForStatus param) {
        Product product = productService.get(param.getId());
        product.setStatus(param.getStatus());
        Product tmp = productService.save(product);
        return ApiDTO.ok("删除成功！", tmp);
    }


    @ApiOperation(value = "分页搜索SN规则信息")
    @PostMapping("/pageQuery")
    public ApiDTO pageQuery(@RequestBody QueryParamForProduct param) {

        List<Product> resultByType = productService.findAll(QueryParamExp.eq("snType", param.getType()));

        resultByType.sort(Comparator.comparing(Product::getAddTime).reversed());
        int resultSize = resultByType.size();
        int startIndex = (param.getIndex() - 1) * param.getLength();
        int endIndex = startIndex + param.getLength();
        List<Product> resultByTypeAndPage;
        try {
            if (endIndex > resultSize) {
                endIndex = resultSize;
                resultByTypeAndPage = resultByType.subList(startIndex, endIndex);
            } else {
                resultByTypeAndPage = resultByType.subList(startIndex, endIndex);
            }
        } catch (IllegalArgumentException e) {
            return ApiDTO.error("分页索引越界！");
        }
        ApiPageDTO result = new ApiPageDTO("搜索成功！", resultByTypeAndPage);
        result.setTotal(resultByType.size());
        return result;
    }

}
