package cn.fastfun.service.impl;

import cn.fastfun.controller.param.RoleTree;
import cn.fastfun.service.SysService;
import cn.fastfun.service.UtilService;
import cn.fastfun.service.entity.SysRole;
import cn.fastfun.service.entity.SysRoleComponent;
import cn.fastfun.service.entity.SysUser;
import cn.fastfun.service.entity.SysUserRole;
import com.bridge.dto.QueryParamExp;
import com.bridge.service.JpaService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import javax.transaction.Transactional;
import java.util.List;

@Slf4j
@Service
public class SysServiceImpl implements SysService {

    @Resource
    UtilService utilService;

    //业务类
    @Resource(name = "sysUserService")
    JpaService<SysUser, String> sysUserService;

    //业务类
    @Resource(name = "sysUserRoleService")
    JpaService<SysUserRole, String> sysUserRoleService;

    //业务类
    @Resource(name = "sysRoleComponentService")
    JpaService<SysRoleComponent, String> sysRoleComponentService;

    @Transactional
    @Override
    public void saveRoleMenu(RoleTree param) {
        List<SysRoleComponent> sysRoleComponents = sysRoleComponentService.findAll(QueryParamExp.eq("roleCode", param.getRoleCode()));
        sysRoleComponents.forEach(p -> sysRoleComponentService.delete(p.getId()));

        param.getIds().forEach(p -> {
            SysRoleComponent obj = new SysRoleComponent();
            obj.setRoleCode(param.getRoleCode());
            obj.setComponentId(p);
            sysRoleComponentService.save(obj);
        });

        // 强制下线相关用户
        List<SysUserRole> sysUserRoles = sysUserRoleService.findAll(QueryParamExp.eq("roleCode", param.getRoleCode()));
        sysUserRoles.forEach(p -> utilService.clearLoginUser(p.getUserId()));
    }

    @Override
    @Transactional
    public void saveUserRole(SysUser param) {

        SysUser entity = sysUserService.save(param);
        List<SysUserRole> userRoles = sysUserRoleService.findAll(QueryParamExp.eq("userId", entity.getId()));
        userRoles.forEach(p -> sysUserRoleService.delete(p));

        if (!CollectionUtils.isEmpty(param.getRoleIds())) {
            param.getRoleIds().forEach(p -> sysUserRoleService.save(new SysUserRole(entity.getId(), p)));
        }


    }
}
